package eu.stork.sp;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts2.interceptor.ServletRequestAware;
import org.apache.struts2.interceptor.ServletResponseAware;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.opensymphony.xwork2.Action;
import com.opensymphony.xwork2.ActionSupport;

import eu.eidas.auth.commons.EidasStringUtil;
import eu.eidas.auth.commons.attribute.AttributeDefinition;
import eu.eidas.auth.commons.attribute.ImmutableAttributeMap;
import eu.eidas.auth.commons.attribute.PersonType;
import eu.eidas.auth.commons.attribute.impl.StringAttributeValueMarshaller;
import eu.eidas.auth.commons.protocol.IRequestMessageNoMetadata;
import eu.eidas.auth.commons.protocol.eidas.LevelOfAssurance;
import eu.eidas.auth.commons.protocol.eidas.LevelOfAssuranceComparison;
import eu.eidas.auth.commons.protocol.eidas.impl.EidasAuthenticationRequestNoMetadata;
import eu.eidas.auth.commons.protocol.impl.EidasSamlBinding;
import eu.eidas.auth.commons.protocol.impl.SamlNameIdFormat;
import eu.eidas.auth.engine.ProtocolEngineNoMetadataI;
import eu.eidas.auth.engine.xml.opensaml.SAMLEngineUtils;
import eu.eidas.auth.engine.xml.opensaml.SecureRandomXmlIdGenerator;
import eu.eidas.engine.exceptions.EIDASSAMLEngineException;

/**
 * 
 * @author iinigo This Action Generates a SAML Request with the data given by
 *         the user, then sends it to the selected peps
 * 
 */

public class IndexAction extends ActionSupport implements ServletRequestAware,
		ServletResponseAware {

	private static final long serialVersionUID = 3660074009157921579L;

	static final Logger logger = LoggerFactory.getLogger(IndexAction.class
			.getName());

	private String SAMLRequest;
	private String samlRequestXML;

	private static Properties configs;
	
	private List<String> proxyUrls;
	private String selectUrl;

	private static String providerName;
	private static String spSector;
	private static String spApplication;

	/* Requested parameters */
	private String pepsUrl;

	private String returnUrl;

	// Checkboxes para excluir mecanismos de autenticación
	private boolean checkAFirma;
	private boolean checkStork;
	private boolean checkSS;
	private boolean checkAEAT;
	private String excludedIdPList;
	private String forcedIdP = "none";
	private String relayState = "";
	
	// Checkboxes para permitir certificados de persona juridica
	private boolean allowLegalPerson = false;
	
	// Checkboxes para Single Sign On
	private boolean forceAuth = true;

	// Lista de IdPs para forzar
	private List<String> opcionesIdP;
	private String selectIdP;

	/**
	 * Fill the data in the JSP that is shown to the user in order to fill the
	 * requested data to generate a saml request
	 */
	public String populate() {

		try {
			configs = SPUtil.loadConfigs(Constants.SP_PROPERTIES);
		} catch (IOException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException(
					"Could not load configuration file", e.getMessage());
		}

		returnUrl = configs.getProperty(Constants.SP_RETURN);
		providerName = configs.getProperty(Constants.PROVIDER_NAME);
		spSector = configs.getProperty(Constants.SP_SECTOR);
		spApplication = configs.getProperty(Constants.SP_APLICATION);
		pepsUrl = selectUrl;

		// Rellenar la lista para forzar un IdP
		opcionesIdP = new ArrayList<String>();
		opcionesIdP.add("@Firma");
		opcionesIdP.add("Stork");
		opcionesIdP.add("Seguridad Social");
		opcionesIdP.add("Agencia Tributaria");

		return "populate";
	}

	/**
	 * Generates the Saml Request with the data given by the user
	 */
	public String execute() {

		byte[] token = null;

		ImmutableAttributeMap.Builder reqAttrMapBuilder = new ImmutableAttributeMap.Builder();
		if (checkAFirma) {
			reqAttrMapBuilder.put(new AttributeDefinition.Builder<String>().nameUri("http://es.minhafp.clave/AFirmaIdP")
					.friendlyName("AFirmaIdP")
					.personType(PersonType.NATURAL_PERSON)
					.required(false)
					.uniqueIdentifier(true)
					.xmlType("http://www.w3.org/2001/XMLSchema", "AFirmaIdPType", "cl")
					.attributeValueMarshaller(new StringAttributeValueMarshaller())
					.build());
		}
		if (checkSS) {
			reqAttrMapBuilder.put(new AttributeDefinition.Builder<String>().nameUri("http://es.minhafp.clave/GISSIdP")
					.friendlyName("GISSIdP")
					.personType(PersonType.NATURAL_PERSON)
					.required(false)
					.uniqueIdentifier(true)
					.xmlType("http://www.w3.org/2001/XMLSchema", "GISSIdPType", "cl")
					.attributeValueMarshaller(new StringAttributeValueMarshaller())
					.build());
		}
		if (checkAEAT) {
			reqAttrMapBuilder.put(new AttributeDefinition.Builder<String>().nameUri("http://es.minhafp.clave/AEATIdP")
					.friendlyName("AEATIdP")
					.personType(PersonType.NATURAL_PERSON)
					.required(false)
					.uniqueIdentifier(true)
					.xmlType("http://www.w3.org/2001/XMLSchema", "AEATIdPType", "cl")
					.attributeValueMarshaller(new StringAttributeValueMarshaller())
					.build());
		}
		if (checkStork) {
			reqAttrMapBuilder.put(new AttributeDefinition.Builder<String>().nameUri("http://es.minhafp.clave/EIDASIdP")
					.friendlyName("EIDASIdP")
					.personType(PersonType.NATURAL_PERSON)
					.required(false)
					.uniqueIdentifier(true)
					.xmlType("http://www.w3.org/2001/XMLSchema", "EIDASIdP", "cl")
					.attributeValueMarshaller(new StringAttributeValueMarshaller())
					.build());
		}
		
		reqAttrMapBuilder.putPrimaryValues(new AttributeDefinition.Builder<String>().nameUri("http://es.minhafp.clave/RelayState")
    			.friendlyName("RelayState")
    			.personType(PersonType.NATURAL_PERSON)
    			.required(false)
    			.uniqueIdentifier(true)
    			.xmlType("http://eidas.europa.eu/attributes/naturalperson", "PersonIdentifierType", "eidas-natural")
    			.attributeValueMarshaller(new StringAttributeValueMarshaller())
    			.build(), SecureRandomXmlIdGenerator.INSTANCE.generateIdentifier(8));

		EidasAuthenticationRequestNoMetadata.Builder reqBuilder = new EidasAuthenticationRequestNoMetadata.Builder();
        
        reqBuilder.destination(pepsUrl);
        reqBuilder.providerName(providerName);
        reqBuilder.requestedAttributes(reqAttrMapBuilder.build());
        reqBuilder.levelOfAssurance(LevelOfAssurance.LOW.stringValue());
        reqBuilder.levelOfAssuranceComparison(LevelOfAssuranceComparison.fromString("minimum").stringValue());
        reqBuilder.nameIdFormat(SamlNameIdFormat.UNSPECIFIED.getNameIdFormat());
        reqBuilder.binding(EidasSamlBinding.POST.getName());
        reqBuilder.assertionConsumerServiceURL(returnUrl);
        reqBuilder.forceAuth(isForceAuth());
        
        reqBuilder.spType(spSector);
        reqBuilder.spApplication(spApplication);
        
        IRequestMessageNoMetadata binaryRequestMessage = null;
        EidasAuthenticationRequestNoMetadata authRequest = null;
        try {
        	reqBuilder.id(SAMLEngineUtils.generateNCName());
            authRequest = reqBuilder.build();
			ProtocolEngineNoMetadataI engine = SPUtil.getProtocolEngine();
			binaryRequestMessage = engine.generateRequestMessage(authRequest);
		} catch (EIDASSAMLEngineException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException(
					"Could not generate token for Saml Request",
					e.getErrorMessage());
		}
        
        token = binaryRequestMessage.getMessageBytes();

		SAMLRequest = EidasStringUtil.encodeToBase64(token);
		samlRequestXML = new String(token);
		relayState = SecureRandomXmlIdGenerator.INSTANCE.generateIdentifier(8);
		
		generateExcludedIdPList();

		return Action.SUCCESS;
	}

	/**
	 * Generates the default Saml Request with eIdentifier, givenName and
	 * surname
	 */
	public String defaultRequest() {
		excludedIdPList = "none";
		forcedIdP = "none";

		try {
			configs = SPUtil.loadConfigs(Constants.SP_PROPERTIES);
		} catch (IOException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException(
					"Could not load configuration file", e.getMessage());
		}

		returnUrl = configs.getProperty(Constants.SP_RETURN);
		providerName = configs.getProperty(Constants.PROVIDER_NAME);
		spApplication = configs.getProperty(Constants.SP_APLICATION);
		pepsUrl = selectUrl;

		byte[] token = null;
		
		ImmutableAttributeMap.Builder reqAttrMapBuilder = new ImmutableAttributeMap.Builder();
        
        reqAttrMapBuilder.putPrimaryValues(new AttributeDefinition.Builder<String>().nameUri("http://es.minhafp.clave/RelayState")
    			.friendlyName("RelayState")
    			.personType(PersonType.NATURAL_PERSON)
    			.required(false)
    			.uniqueIdentifier(true)
    			.xmlType("http://eidas.europa.eu/attributes/naturalperson", "PersonIdentifierType", "eidas-natural")
    			.attributeValueMarshaller(new StringAttributeValueMarshaller())
    			.build(), SecureRandomXmlIdGenerator.INSTANCE.generateIdentifier(8));
        
        EidasAuthenticationRequestNoMetadata.Builder reqBuilder = new EidasAuthenticationRequestNoMetadata.Builder();
        
        reqBuilder.destination(pepsUrl);
        reqBuilder.providerName(providerName);
        reqBuilder.requestedAttributes(reqAttrMapBuilder.build());
        reqBuilder.levelOfAssurance(LevelOfAssurance.LOW.stringValue());
        reqBuilder.levelOfAssuranceComparison(LevelOfAssuranceComparison.fromString("minimum").stringValue());
        reqBuilder.nameIdFormat(SamlNameIdFormat.UNSPECIFIED.getNameIdFormat());
        reqBuilder.binding(EidasSamlBinding.POST.getName());
        reqBuilder.assertionConsumerServiceURL(returnUrl);
        reqBuilder.forceAuth(forceAuth);
        
        reqBuilder.spApplication(spApplication);
        
        IRequestMessageNoMetadata binaryRequestMessage = null;
        EidasAuthenticationRequestNoMetadata authRequest = null;
        try {
        	reqBuilder.id(SAMLEngineUtils.generateNCName());
            authRequest = reqBuilder.build();
			ProtocolEngineNoMetadataI engine = SPUtil.getProtocolEngine();
			binaryRequestMessage = engine.generateRequestMessage(authRequest);
		} catch (EIDASSAMLEngineException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException(
					"Could not generate token for Saml Request",
					e.getErrorMessage());
		}

		token = binaryRequestMessage.getMessageBytes();

		SAMLRequest = EidasStringUtil.encodeToBase64(token);
		samlRequestXML = new String(token);
		relayState = SecureRandomXmlIdGenerator.INSTANCE.generateIdentifier(8);

		return "default";
	}
		
	private void generateExcludedIdPList() {
		excludedIdPList = new String();
		if (selectIdP != null) {
			if (selectIdP.equals("none")) {
				if (checkAFirma)
					excludedIdPList = excludedIdPList.concat("aFirma;");
				if (checkStork)
					excludedIdPList = excludedIdPList.concat("Stork;");	
				if (checkSS)
					excludedIdPList = excludedIdPList.concat("SS;");
				if (checkAEAT)
					excludedIdPList = excludedIdPList.concat("AEAT;");
				if (excludedIdPList.isEmpty())
					excludedIdPList = "none";
			} else {
				if (selectIdP.equals("@Firma"))
					forcedIdP = "aFirma";
				if (selectIdP.equals("Stork"))
					forcedIdP = "Stork";
				if (selectIdP.equals("Agencia Tributaria"))
					forcedIdP = "AEAT";
				if (selectIdP.equals("Seguridad Social"))
					forcedIdP = "SS";
				excludedIdPList = "none";
			}
		}
	}

	public String loadProxyUrls() {
		try {
			configs = SPUtil.loadConfigs(Constants.SP_PROPERTIES);
		} catch (IOException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException(
					"Could not load configuration file", e.getMessage());
		}
		
		int proxyNumber = Integer.parseInt(configs.getProperty(Constants.COUNTRY_URL_NUMBER));
		proxyUrls = new ArrayList<String>();
		
		for (int i=0; i<proxyNumber; i++) {
			proxyUrls.add(configs.getProperty(Constants.COUNTRY_URL + (i+1)));
		}
		
		return Action.SUCCESS;	
	}


	public void setSAMLRequest(String samlToken) {
		this.SAMLRequest = samlToken;
	}

	public String getSAMLRequest() {
		return SAMLRequest;
	}

	public String getProviderName() {
		return providerName;
	}

	public void setProviderName(String provName) {
		providerName = provName;
	}

	public String getSelectUrl() {
		return selectUrl;
	}

	public void setSelectUrl(String selectUrl) {
		this.selectUrl = selectUrl;
	}

	public List<String> getProxyUrls() {
		return proxyUrls;
	}

	public void setProxyUrls(List<String> proxyUrls) {
		this.proxyUrls = proxyUrls;
	}

	public String getSamlRequestXML() {
		return samlRequestXML;
	}

	public void setSamlRequestXML(String samlRequestXML) {
		this.samlRequestXML = samlRequestXML;
	}

	public String getReturnUrl() {
		return returnUrl;
	}

	public void setReturnUrl(String returnUrl) {
		this.returnUrl = returnUrl;
	}

	public String getPepsUrl() {
		return pepsUrl;
	}

	public void setPepsUrl(String pepsUrl) {
		this.pepsUrl = pepsUrl;
	}

	public void setServletRequest(HttpServletRequest request) {
	}

	public void setServletResponse(HttpServletResponse response) {
	}

	public boolean isCheckAFirma() {
		return checkAFirma;
	}

	public void setCheckAFirma(boolean checkAFirma) {
		this.checkAFirma = checkAFirma;
	}

	public boolean isCheckStork() {
		return checkStork;
	}

	public void setCheckStork(boolean checkStork) {
		this.checkStork = checkStork;
	}

	public boolean isCheckSS() {
		return checkSS;
	}

	public String getExcludedIdPList() {
		return excludedIdPList;
	}

	public void setExcludedIdPList(String excludedIdPList) {
		this.excludedIdPList = excludedIdPList;
	}

	public String getForcedIdP() {
		return forcedIdP;
	}

	public void setForcedIdP(String forcedIdP) {
		this.forcedIdP = forcedIdP;
	}

	public String getRelayState() {
		return relayState;
	}

	public void setRelayState(String relayState) {
		this.relayState = relayState;
	}

	public boolean isAllowLegalPerson() {
		return allowLegalPerson;
	}

	public void setAllowLegalPerson(boolean allowLegalPerson) {
		this.allowLegalPerson = allowLegalPerson;
	}

	public boolean isForceAuth() {
		return forceAuth;
	}

	public void setForceAuth(boolean forceAuth) {
		this.forceAuth = forceAuth;
	}

	public void setCheckSS(boolean checkSS) {
		this.checkSS = checkSS;
	}

	public boolean isCheckAEAT() {
		return checkAEAT;
	}

	public void setCheckAEAT(boolean checkAEAT) {
		this.checkAEAT = checkAEAT;
	}

	public List<String> getOpcionesIdP() {
		return opcionesIdP;
	}

	public void setOpcionesIdP(List<String> opcionesIdP) {
		this.opcionesIdP = opcionesIdP;
	}

	public String getSelectIdP() {
		return selectIdP;
	}

	public void setSelectIdP(String selectIdP) {
		this.selectIdP = selectIdP;
	}
}
