package eu.stork.sp;

import java.io.IOException;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts2.interceptor.ServletRequestAware;
import org.opensaml.saml2.core.LogoutResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.opensymphony.xwork2.Action;
import com.opensymphony.xwork2.ActionSupport;

import eu.eidas.auth.commons.EidasStringUtil;
import eu.eidas.auth.engine.ProtocolEngineNoMetadataI;
import eu.eidas.auth.engine.xml.opensaml.SecureRandomXmlIdGenerator;
import eu.eidas.engine.exceptions.EIDASSAMLEngineException;

public class LogoutAction extends ActionSupport implements ServletRequestAware {

	private static final long serialVersionUID = 3660074009157921579L;

	static final Logger logger = LoggerFactory.getLogger(LogoutAction.class
			.getName());
	
	private HttpServletRequest request;

	private String samlRequestLogout;
	
	private String relayState = "";
	
	private String logoutResponse;

	private static Properties configs;
	
	private String proxySelected;

	private String proxyUrl;

	private String homepage;
	
	@Override
	public void setServletRequest(HttpServletRequest arg0) {
		request = arg0;
	}

	/**
	 * Genera una solicitud de Logout
	 */
	public String execute() {
		
		try {
			configs = SPUtil.loadConfigs(Constants.SP_PROPERTIES);
		} catch (IOException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException(
					"No se ha podido cargar el fichero de configuraciones", e.getMessage());
		}
		
		ProtocolEngineNoMetadataI engine = SPUtil.getProtocolEngine();

		try {			
			final byte[] token = engine.generateLogoutRequestMessage(configs.getProperty(Constants.LOGOUT_RESPONSE_URL),
					configs.getProperty(Constants.PROVIDER_NAME),
					proxySelected + configs.getProperty(Constants.LOGOUT_CPEPS_URL),
					SecureRandomXmlIdGenerator.INSTANCE.generateIdentifier(8));

			samlRequestLogout = EidasStringUtil.encodeToBase64(token);
			relayState = SecureRandomXmlIdGenerator.INSTANCE.generateIdentifier(8);
			
			proxyUrl = proxySelected + configs.getProperty(Constants.LOGOUT_CPEPS_URL);

		} catch (EIDASSAMLEngineException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException("Could not generate token for Saml Logout Request",
					e.getErrorMessage());
		}
		return Action.SUCCESS;
	}
	
	/**
	 * Valida la respuesta
	 */
	public String validateResponse() throws Exception {	
		try {
			configs = SPUtil.loadConfigs(Constants.SP_PROPERTIES);
		} catch (IOException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException(
					"No se ha podido cargar el fichero de configuraciones", e.getMessage());
		}
		
		// Se valida la respuesta
		LogoutResponse logr = processLogoutResponse(EidasStringUtil.decodeBytesFromBase64(logoutResponse),
				request.getRemoteHost());
		
		setHomepage(configs.getProperty("sp.url"));
		
		// A continuación iría la logica de negocio específica del SP
		
		logger.info("Logout response obtained: " + logr.getStatus());
		
		return "loggedout";
	}
	
	private LogoutResponse processLogoutResponse(byte[] samlToken, String remoteHost) {

		// validates SAML Token
		try {
			ProtocolEngineNoMetadataI engine = SPUtil.getProtocolEngine();
			final LogoutResponse logoutReq = engine.unmarshallLogoutResponseAndValidate(samlToken,
					remoteHost, 0, 0, configs.getProperty(Constants.SP_RETURN));
			return logoutReq;
		} catch (EIDASSAMLEngineException e) {
			LOG.error("Error validating SAMLToken", e);
			return null;
		}
	}

	public String getProxyUrl() {
		return proxyUrl;
	}

	public void setProxyUrl(String proxyUrl) {
		this.proxyUrl = proxyUrl;
	}

	public String getSamlRequestLogout() {
		return samlRequestLogout;
	}

	public void setSamlRequestLogout(String samlRequestLogout) {
		this.samlRequestLogout = samlRequestLogout;
	}

	public String getRelayState() {
		return relayState;
	}

	public void setRelayState(String relayState) {
		this.relayState = relayState;
	}

	public String getLogoutResponse() {
		return logoutResponse;
	}

	public void setLogoutResponse(String samlResponseLogout) {
		this.logoutResponse = samlResponseLogout;
	}

	public String getProxySelected() {
		return proxySelected;
	}

	public void setProxySelected(String proxySelected) {
		this.proxySelected = proxySelected;
	}

	/**
	 * @return the homepage
	 */
	public String getHomepage() {
		return homepage;
	}

	/**
	 * @param homepage the homepage to set
	 */
	public void setHomepage(String homepage) {
		this.homepage = homepage;
	}
}
