package eu.stork.sp;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts2.interceptor.ServletRequestAware;
import org.apache.struts2.interceptor.ServletResponseAware;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSet;
import com.google.common.collect.UnmodifiableIterator;
import com.opensymphony.xwork2.Action;
import com.opensymphony.xwork2.ActionSupport;

import eu.eidas.auth.commons.EidasStringUtil;
import eu.eidas.auth.commons.attribute.AttributeDefinition;
import eu.eidas.auth.commons.attribute.AttributeValue;
import eu.eidas.auth.commons.protocol.IAuthenticationResponseNoMetadata;
import eu.eidas.auth.engine.ProtocolEngineNoMetadataI;
import eu.eidas.engine.exceptions.EIDASSAMLEngineException;

/**
 * This Action recives a SAML Response, shows it to the user and then validates
 * it getting the attributes values
 * 
 * @author iinigo
 * 
 */
public class ReturnAction extends ActionSupport implements ServletRequestAware,
		ServletResponseAware {

	private static final long serialVersionUID = 3660074009157921579L;

	static final Logger logger = LoggerFactory.getLogger(IndexAction.class
			.getName());

	private String SAMLResponse;
	private String samlResponseXML;
	private String relayState = "";

	private ImmutableMap<AttributeDefinition<?>, ImmutableSet<? extends AttributeValue<?>>> attrList = null;
	
	private ArrayList<String> attrListString = new ArrayList<String>();
	private ArrayList<String> attrListValString  = new ArrayList<String>();

	private String scope;

	private HttpServletRequest request;
	
	private static Properties configs;

	private static String providerName;
	private static String homepage = null;

	/**
	 * Translates the SAMLResponse to XML format in order to be shown in the JSP
	 * 
	 * @return
	 */
	public String execute() {

		try {
			configs = SPUtil.loadConfigs(Constants.SP_PROPERTIES);
		} catch (IOException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException(
					"No se ha podido cargar el fichero de configuraciones", e.getMessage());
		}

		providerName = configs.getProperty(Constants.PROVIDER_NAME);

		byte[] decSamlToken = EidasStringUtil.decodeBytesFromBase64(SAMLResponse);
		samlResponseXML = new String(decSamlToken);

		return Action.SUCCESS;
	}

	/**
	 * Validates the response and displays the value of the requested attributes
	 * 
	 * @return
	 */
	public String populate() {
		
		try {
			configs = SPUtil.loadConfigs(Constants.SP_PROPERTIES);
		} catch (IOException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException(
					"No se ha podido cargar el fichero de configuraciones", e.getMessage());
		}
		
		homepage = configs.getProperty("sp.url");

		IAuthenticationResponseNoMetadata authnResponse = null;

		// Decodes SAML Response
		byte[] decSamlToken = EidasStringUtil.decodeBytesFromBase64(SAMLResponse);

		// Get SAMLEngine instance
		ProtocolEngineNoMetadataI engine = SPUtil.getProtocolEngine();

		try {
			// validate SAML Token
			authnResponse = engine.unmarshallResponseAndValidate(decSamlToken,
					(String) request.getRemoteHost(), 0, 0, 
					configs.getProperty(Constants.SP_RETURN));
			//TODO: Comprobar la consistencia del RelayState enviado y recibido.
		} catch (EIDASSAMLEngineException e) {
			logger.error(e.getMessage());
			throw new ApplicationSpecificServiceException("No se ha podido validar el token de la SAML Response", e.getErrorMessage());
		}			
		
		if (authnResponse.isFailure()) {
			throw new ApplicationSpecificServiceException("La SAML Response ha dado un fallo", authnResponse.getStatusMessage());			
		} else {			
			//Get attributes
			attrList = authnResponse.getAttributes().getAttributeMap();
			UnmodifiableIterator<AttributeDefinition<?>> iterator = attrList.keySet().iterator();
			while (iterator.hasNext()) {
				AttributeDefinition<?> it = iterator.next();
				attrListString.add(it.getFriendlyName());
				attrListValString.add(attrList.get(it).toString());
			}
			
			if (authnResponse.getIssuer() != null){
				scope = authnResponse.getIssuer();
				
				switch(scope.substring(0, 6)) {
					case Constants.SCOPE_AFIRMA:
						scope = "DNIe / Certificado electrónico";
					break;
					case Constants.SCOPE_AEAT:
						scope = "Cl@ve PIN";
					break;
					case Constants.SCOPE_SS:
						scope = "Cl@ve permanente";
					break;
					case Constants.SCOPE_STORK + "-":
						scope = "Ciudadanos UE - " + scope.substring(6);
				}
			}

			return "populate";
		}
	}

	public void setServletRequest(HttpServletRequest request) {
		this.request = request;
	}

	public void setServletResponse(HttpServletResponse response) {
	}

	public ImmutableMap<AttributeDefinition<?>, ImmutableSet<? extends AttributeValue<?>>> getAttrList() {
		return attrList;
	}

	public ArrayList<String> getAttrListString() {
		return attrListString;
	}

	public void setAttrListString(ArrayList<String> attrListString) {
		this.attrListString = attrListString;
	}

	public ArrayList<String> getAttrListValString() {
		return attrListValString;
	}

	public void setAttrListValString(ArrayList<String> attrListValString) {
		this.attrListValString = attrListValString;
	}

	public String getScope() {
		return scope;
	}

	public void setScope(String scope) {
		this.scope = scope;
	}

	public String getProviderName() {
		return providerName;
	}

	public void setProviderName(String provName) {
		providerName = provName;
	}

	public String getHomepage() {
		return homepage;
	}

	public void setHomepage(String homepage) {
		ReturnAction.homepage = homepage;
	}

	public String getSAMLResponse() {
		return SAMLResponse;
	}

	public void setSAMLResponse(String samlResponse) {
		this.SAMLResponse = samlResponse;
	}

	public String getRelayState() {
		return relayState;
	}

	public void setRelayState(String relayState) {
		this.relayState = relayState;
	}

	public String getSamlResponseXML() {
		return samlResponseXML;
	}

	public void setSamlResponseXML(String samlResponseXML) {
		this.samlResponseXML = samlResponseXML;
	}

}